#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/5/10 9:48
# @Author  : chenjw
# @Site    : 
# @File    : tapd_story.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

from urllib.parse import urlencode
from ccommon.httpSimple import HttpSimple
from tapdApi import tapd, tapd_param
import datetime


class TapdStorys(tapd.Tapd):
    def __init__(self, usr, pwd, delay=1000):
        tapd.Tapd.__init__(self, usr, pwd, delay)

    def storyChange(self, workspace_id, story_id, creator=None, created=None, order=None, limit=None, page=None,
                    fields=None):
        '''
        :param workspace_id:    N       int         所属 TAPD 项目 ID
        :param story_id:        N       int         Story ID(story_id=1,2,3 支持一次性读取最多50 个 story_id 的数据)
        :param creator:         O       str         变更人
        :param created:         O       datetime    变更时间
        :param order:           O       str         按照上面的字段进行排序
        :param limit:           O       int         设置返回数量限制，默认为 30
        :param page:            O       int         返回当前数量限制下第 N 页的数据，默认为 1(第一页）
        :param fields:          O       str         设置获取的字段，多个字段间以','逗号隔开
        :return:
        '''
        url = 'https://api.tapd.cn/story_changes?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(story_id, int, 'story_id')
        param.setValue(creator, str, 'creator')
        # param.setValue(created, datetime.datetime, 'created')
        param.setValue(order, str, 'order')
        param.setValue(limit, int, 'limit')
        param.setValue(page, int, 'page')
        param.setValue(fields, str, 'fields')
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        if isinstance(created, dict):
            btime = None
            etime = None
            if 'btime' in created.keys():
                btime = created['btime']
            if 'etime' in created.keys():
                etime = created['etime']
            if btime is not None and etime is not None:
                url += '&created=%s~%s' % (btime, etime)
            elif btime is not None:
                url += '&created=>%s' % (btime)
            elif etime is not None:
                url += '&created=<%s' % (etime)

        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()

        self.checkApiStatus(_json)
        return [ain_json.Dict for ain_json in _json.AryJson('data')]

    def listStory(self, workspace_id, id=None, name=None, creator=None, status=None, description=None, owner=None,
                  cc=None, size=None, priority=None, created=None, begin=None, due=None, modified=None,
                  iteration_id=None, developer=None, test_focus=None, type=None, source=None, version=None,
                  category_id=None, business_value=None, limit=None, page=None, fields=None):
        '''
        :param workspace_id:        N       int         项目 ID
        :param id:                  O       int         需求的 ID
        :param name:                O       str         需求标题
        :param creator:             O       str         创建人
        :param status:              O       str         状态
        :param description:         O       str         需求描述
        :param owner:               O       str         当前处理人
        :param cc:                  O       str         抄送人
        :param size:                O       int         规模
        :param priority:            O       str         优先级
        :param created:             O       datetime    创建时间
        :param begin:               O       datetime    预计开始时间
        :param due:                 O       datetime    预计结束时间
        :param modified:            O       datetime    最后修改时间
        :param iteration_id:        O       str         迭代 id
        :param developer:           O       str         开发者
        :param test_focus:          O       str         测试点
        :param type:                O       str         类型
        :param source:              O       str         来源
        :param version:             O       str         版本
        :param category_id:         O       str         模块
        :param business_value:      O       str         分类 id
        :param limit:               O       int         设置返回数量限制，默认为 30
        :param page:                O       int         返回当前数量限制下第 N 页的数据，默认为 1（第一页）
        :param fields:              O       str         设置获取的字段，多个字段间以','逗号隔开
        :return:
        '''
        url = 'https://api.tapd.cn/stories?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(id, int, 'id')
        param.setValue(name, str, 'name')
        param.setValue(creator, str, 'creator')
        param.setValue(status, str, 'status')
        param.setValue(description, str, 'description')
        param.setValue(owner, str, 'owner')
        param.setValue(cc, str, 'cc')
        param.setValue(size, int, 'size')
        param.setValue(priority, str, 'priority')
        param.setValue(created, datetime.datetime, 'created')
        param.setValue(begin, datetime.datetime, 'begin')
        param.setValue(due, datetime.datetime, 'due')
        param.setValue(modified, datetime.datetime, 'modified')
        param.setValue(iteration_id, str, 'iteration_id')
        param.setValue(developer, str, 'developer')
        param.setValue(test_focus, str, 'test_focus')
        param.setValue(type, str, 'type')
        param.setValue(source, str, 'source')
        param.setValue(version, str, 'version')
        param.setValue(category_id, str, 'category_id')
        param.setValue(business_value, str, 'business_value')
        param.setValue(limit, int, 'limit')
        param.setValue(page, int, 'page')
        param.setValue(fields, str, 'fields')
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))
        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        if _json.ExistP('data/Story/id'):
            return _json.JsonP('data/Story').Dict
        elif _json.ExistP('data/0/Story/id'):
            return [ain_json.Dict for ain_json in _json.AryJson('data')]
        else:
            return None

    def statusMap(self, workspace_id):
        '''
        :param workspace_id:        N       int         项目 ID
        :return:
        '''
        system = 'story'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(system, str, 'system')
        url = 'https://api.tapd.cn/workflows/status_map?'
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))

        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()

        self.checkApiStatus(_json)
        return _json.Json('data').Dict


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/23 9:55
# @Author  : chenjw
# @Site    : 
# @File    : jsonParse.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

import json
from ccommon import util


class JsonParse:
    def __init__(self, target=None):
        if target is None:
            self.Dict = {}
        elif isinstance(target, str):
            self.Dict = json.loads(target)
        elif isinstance(target, dict):
            self.Dict = json.loads(json.dumps(target))
        else:
            self.Dict = {}

    def isEmpty(self):
        if len(self.Dict) == 0:
            return True
        return False

    def Exist(self, key):
        return self.exist(key, self.Dict)

    # inner func
    def exist(self, key, default_dict):
        if isinstance(key, str) is False:
            return False
        if key in default_dict.keys():
            return True
        else:
            return False

    def Val(self, key):
        return self.val(key, self.Dict)

    # inner func
    def val(self, key, default_dict):
        if self.exist(key, default_dict) is False:
            return None
        else:
            return default_dict[key]

    def ExistP(self, keyP):
        if isinstance(keyP, str) is False:
            return False
        return self.existP(keyP, self.Dict)

    # inner func
    def isInt(self, s):
        try:
            if int(s) >= 0:
                return True
            return False
        except:
            return False

    # inner func
    def existP(self, keyP, default_dict=None, default_list=None):
        if default_dict is None and default_list is None:
            return False
        if keyP == '':
            return False
        index = keyP.find('/')
        if index < 0:
            if default_dict is not None:
                return self.exist(keyP, default_dict)
            else:
                return False
        elif index == 0:
            if len(keyP) > 1:
                return self.existP(keyP[1:], default_dict, default_list)
            else:
                return False
        elif index == len(keyP) - 1:
            return self.existP(keyP[:-1], default_dict, default_list)
        else:
            pre = keyP[:index]
            suffix = keyP[index + 1:]
            if isinstance(default_dict, dict) and self.exist(pre, default_dict):
                tmp = default_dict[pre]
                if isinstance(tmp, dict):
                    return self.existP(suffix, tmp, None)
                elif isinstance(tmp, list):
                    return self.existP(suffix, None, tmp)
                return False
            elif self.isInt(pre) and isinstance(default_list, list) and int(pre) < len(default_list) and isinstance(
                    default_list[int(pre)], dict):
                return self.existP(suffix, default_list[int(pre)], None)
            else:
                return False

    def ValP(self, keyP):
        if self.ExistP(keyP.strip()) is False:
            return None
        return self.valP(keyP.strip(), self.Dict)

    # inner func
    def valP(self, keyP, default_dict=None, default_list=None):
        if default_dict is None and default_list is None:
            return None
        if keyP == '':
            return None
        index = keyP.find('/')
        if index < 0:
            if default_dict is not None and self.exist(keyP, default_dict):
                return default_dict[keyP]
            else:
                return None
        elif index == 0:
            if len(keyP) > 1:
                return self.valP(keyP[1:], default_dict, default_list)
            else:
                return None
        elif index == len(keyP) - 1:
            return self.valP(keyP[:-1], default_dict, default_list)
        else:
            pre = keyP[:index]
            suffix = keyP[index + 1:]
            if isinstance(default_dict, dict) and self.exist(pre, default_dict):
                tmp = default_dict[pre]
                if isinstance(tmp, dict):
                    return self.valP(suffix, tmp, None)
                elif isinstance(tmp, list):
                    return self.valP(suffix, None, tmp)
                return None
            elif self.isInt(pre) and isinstance(default_list, list) and int(pre) < len(default_list) and isinstance(
                    default_list[int(pre)], dict):
                return self.valP(suffix, default_list[int(pre)], None)
            else:
                return None

    def Ary(self, key):
        return self.ary(key, self.Dict)

    # inner func
    def ary(self, key, default_dict):
        if self.exist(key, default_dict) is False:
            return None
        else:
            tmp = default_dict[key]
            if isinstance(tmp, list):
                return tmp
            else:
                return None

    def AryP(self, keyP):
        if self.ExistP(keyP) is False:
            return None
        return self.aryP(keyP, self.Dict, None)

    # inner func
    def aryP(self, keyP, default_dict=None, default_list=None):
        if default_dict is None and default_list is None:
            return None
        if keyP == '':
            return None
        index = keyP.find('/')
        if index < 0:
            if default_dict is not None and self.exist(keyP, default_dict):
                return self.ary(keyP, default_dict)
            else:
                return None
        elif index == 0:
            if len(keyP) > 1:
                return self.aryP(keyP[1:], default_dict, default_list)
            else:
                return None
        elif index == len(keyP) - 1:
            return self.aryP(keyP[:-1], default_dict, default_list)
        else:
            pre = keyP[:index]
            suffix = keyP[index + 1:]
            if isinstance(default_dict, dict) and self.exist(pre, default_dict):
                tmp = default_dict[pre]
                if isinstance(tmp, dict):
                    return self.aryP(suffix, tmp, None)
                elif isinstance(tmp, list):
                    return self.aryP(suffix, None, tmp)
                return None
            elif self.isInt(pre) and isinstance(default_list, list) and int(pre) < len(default_list) and isinstance(
                    default_list[int(pre)], dict):
                return self.aryP(suffix, default_list[int(pre)], None)
            else:
                return None

    def Int(self, key):
        try:
            return int(self.Val(key))
        except:
            return 0

    def IntP(self, key):
        try:
            return int(self.ValP(key))
        except:
            return 0

    def Float(self, key, r=10):
        try:
            return util.round(float(self.Val(key)), r)
        except:
            return 0

    def FloatP(self, key, r=10):
        try:
            return util.round(float(self.ValP(key)), r)
        except:
            return 0

    def Str(self, key):
        try:
            return str(self.Val(key))
        except:
            return ''

    def StrP(self, key):
        try:
            return str(self.ValP(key))
        except:
            return ''

    def Json(self, key):
        try:
            return JsonParse(self.Val(key))
        except:
            return JsonParse({})

    def JsonP(self, key):
        try:
            return JsonParse(self.ValP(key))
        except:
            return JsonParse({})

    def toInt(self, default_list=None):
        if isinstance(default_list, list) is False:
            return None
        int_list = []
        for sin_item in default_list:
            if isinstance(sin_item, int):
                int_list.append(sin_item)
            else:
                return None
        return int_list

    def AryInt(self, key):
        try:
            return list(self.toInt(self.Ary(key)))
        except:
            return []

    def AryIntP(self, key):
        try:
            return list(self.toInt(self.AryP(key)))
        except:
            return []

    def toFloat(self, default_list=None):
        if isinstance(default_list, list) is False:
            return None
        float_list = []
        for sin_item in default_list:
            if isinstance(sin_item, float) or isinstance(sin_item, int):
                float_list.append(float(sin_item))
            else:
                return None
        return float_list

    def AryFloat(self, key):
        try:
            return list(self.toFloat(self.Ary(key)))
        except:
            return []

    def AryFloatP(self, key):
        try:
            return list(self.toFloat(self.AryP(key)))
        except:
            return []

    def toStr(self, default_list=None):
        if isinstance(default_list, list) is False:
            return None
        str_list = []
        for sin_item in default_list:
            if isinstance(sin_item, str):
                str_list.append(sin_item)
            else:
                return None
        return str_list

    def AryStr(self, key):
        try:
            return list(self.toStr(self.Ary(key)))
        except:
            return []

    def AryStrP(self, key):
        try:
            return list(self.toStr(self.AryP(key)))
        except:
            return []

    def toJson(self, default_list=None):
        if isinstance(default_list, list) is False:
            return None
        json_list = []
        for sin_item in default_list:
            if isinstance(sin_item, dict):
                json_list.append(JsonParse(sin_item))
            else:
                return None
        return json_list

    def AryJson(self, key):
        try:
            return list(self.toJson(self.Ary(key)))
        except:
            return []

    def AryJsonP(self, key):
        try:
            return list(self.toJson(self.AryP(key)))
        except:
            return []

    def toString(self):
        return json.dumps(self.Dict)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/23 9:56
# @Author  : chenjw
# @Site    : 
# @File    : html.py
# @Software: PyCharm Community Edition
# @Desc    :  do what
from base64 import b64encode


class Html:
    def __init__(self):
        self.color_red = 'color_red'
        self.color_green = 'color_green'
        self.color_blue = 'color_blue'
        self.color_grey = 'color_grey'
        self.font_b = 'b'
        # here to init different class to formate the style
        self.default_css = '''
        .table1{
            line-height: 2em;
            font-family: Arial;
            border-collapse: separate;
            border-color: #BDB76B;
        }
        .thead_tr {
            color: #000000;
            background-color: #FFFAFA;
            border-bottom: 1px solid #BDB76B;
        }
        .tbody_tr {
            color: #000000;
            font-size: 0.8em;
            background-color: #F8F8FF;
            border-bottom: 1px solid #BDB76B;
        }
        .tbody_tr:hover {
            background-color: #FFFF33;
        }
        .th1 {
            font-weight: normal;
            text-align: left;
            padding: 0 10px;
            text-align: center;
        }
        .color_red{
            background-color: #FFDAB9;
        }
        .color_green{
            background-color: #9AFF9A;
        }
        .color_blue{
            background-color: #7B68EE;
        }
        .color_grey{
            background-color: #CFCFCF;
        }
        
        .font_color_red{
            color: #FFDAB9;
        }
        .font_color_green{
            color: #9AFF9A;
        }
        .font_color_blue{
            color: #7B68EE;
        }
        .font_color_grey{
            color: #CFCFCF;
        }
        '''
        self.default_title = '测试报告'
        self.body_context = ''
        self.total_msg = ''
        pass

    def setTitle(self, title):
        '''
        :param title: name of the whole html
        :return:
        '''
        self.default_title = title
        return self

    def extraColor(self, x, y, extra=None):
        '''
        :param x: index x
        :param y: index y
        :param extra: list
        :return: None or str(color_red or color_blue or ...)
        '''
        if extra is None:
            return None
        color_list = [self.color_blue, self.color_green, self.color_grey, self.color_red]
        if isinstance(extra, list) is True:
            for sin_item in extra:
                if len(sin_item) >= 3 and \
                        isinstance(sin_item[0], int) and \
                        isinstance(sin_item[1], int) and \
                                sin_item[0] == x and \
                                sin_item[1] == y:
                    if isinstance(sin_item[2], str) and sin_item[2] in color_list:
                        return sin_item[2]
                    if isinstance(sin_item[2], list) or isinstance(sin_item[2], tuple):
                        for sin_color in sin_item[2]:
                            if sin_color in color_list:
                                return sin_color

        else:
            return None

    def extraB(self, x, y, extra=None):
        '''
        :param x: index x
        :param y: index y
        :param extra: list
        :return: None or str(b)
        '''
        if extra is None:
            return None
        font_list = [self.font_b]
        if isinstance(extra, list) is True:
            for sin_item in extra:
                if len(sin_item) >= 3 and \
                        isinstance(sin_item[0], int) and \
                        isinstance(sin_item[1], int) and \
                                sin_item[0] == x and \
                                sin_item[1] == y:
                    if isinstance(sin_item[2], str) and sin_item[2] in font_list:
                        return sin_item[2]
                    if isinstance(sin_item[2], list) or isinstance(sin_item[2], tuple):
                        for sin_font in sin_item[2]:
                            if sin_font in font_list:
                                return sin_font

        else:
            return None

    def setTable(self, header, detail, extra=None):
        '''
        :param header: list like ['a','b','c']
        :param detail: 2D list like [['1','2','3'],['4','5','6']]
        :param extra: list like [(0,0,'color_red'),(0,0,'b')] or [(0,0,['color_red','b'])]
        :return:
        '''
        if isinstance(header, list) is False or isinstance(detail, list) is False:
            return self
        header_msg = ''
        for sin_header in header:
            header_msg += '''<th class="th1">%s</th>\n''' % sin_header
        header_msg = '''<thead><tr class="thead_tr th1">''' + header_msg + '''</tr></thead>'''
        detail_msg = ''
        for index_y in range(len(detail)):
            sin_line = detail[index_y]
            tmp_msg = ''
            for index_x in range(len(sin_line)):
                sin_cell = sin_line[index_x]
                b = self.extraB(index_x, index_y, extra)
                if b is not None:
                    sin_cell = '<b>%s</b>' % sin_cell
                color = self.extraColor(index_x, index_y, extra)
                if color is None:
                    tmp_msg += '''<td class="th1">%s</td>\n''' % sin_cell
                else:
                    tmp_msg += '''<td class="th1 %s">%s</td>\n''' % (color, sin_cell)
            detail_msg += '''<tr class="tbody_tr th1">''' + tmp_msg + '''</tr>'''
        detail_msg = '''<tbody>''' + detail_msg + '''</tbody>'''
        self.body_context += '''<table class="table1">''' + header_msg + detail_msg + '''</table>'''
        return self

    def setH(self, desc, index=6, color=None):
        '''
        :param desc: desc that to show the msg
        :param index: size
        :return:
        '''
        if color is None:
            self.body_context += '''<h%d>%s</h%d>\n''' % (index, desc, index)
        else:
            self.body_context += '''<h%d class='font_%s'>%s</h%d>\n''' % (index, color, desc, index)
        return self

    def draw(self, x_names=range(10), l_range=[0, 10], r_range=None, x_name=None, l_name=None, r_name='', title=None,
             l_compare=[], r_compare=[], file_name=None, dpi=100, font=None, markerSize=7):
        import matplotlib
        matplotlib.use('Agg')
        import matplotlib.pyplot as plt
        if font is not None:
            plt.rcParams['font.family'] = [font]  # 用来正常显示中文标签
        plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号

        fig = plt.figure()
        ax = fig.add_subplot(111)
        if r_range is not None or r_compare is not None:
            ax2 = ax.twinx()
        l_lns = []
        l_markers = ['o', 's', '>', 'x']
        for _index in range(len(l_compare)):
            sin_l_compare = l_compare[_index]
            marker = l_markers[_index % len(l_markers)]
            data = sin_l_compare['data']
            label = sin_l_compare['label']
            color = sin_l_compare['color']
            lns = ax.plot(x_names, data, color, label=label, marker=marker, markerSize=markerSize)
            l_lns.append(lns)

        r_lns = []
        r_markers = ['x', '>', 's', 'o']
        for _index in range(len(r_compare)):
            sin_r_compare = r_compare[_index]
            marker = r_markers[_index % len(r_markers)]
            data = sin_r_compare['data']
            label = sin_r_compare['label']
            color = sin_r_compare['color']
            lns = ax2.plot(x_names, data, color, label=label, marker=marker, markerSize=markerSize)
            r_lns.append(lns)

        def add_lns(lns, lns_list):
            for sin_lns in lns_list:
                if lns is None:
                    lns = sin_lns
                else:
                    lns += sin_lns
            return lns

        lns = None
        lns = add_lns(lns, l_lns)
        lns = add_lns(lns, r_lns)
        labs = [l.get_label() for l in lns]

        ax.legend(lns, labs, loc=0)
        ax.grid()

        if x_name is not None:
            ax.set_xlabel(x_name)

        if l_range is not None:
            ax.set_ylim(l_range[0], l_range[1])

        if l_name is not None:
            ax.set_ylabel(l_name)
        if title is not None:
            ax.set_title(title)

        if r_range is not None:
            ax2.tick_params(axis='y', colors='red')  # 刻度颜色
            ax2.spines['right'].set_color('red')  # 纵轴颜色
            ax2.set_ylim(r_range[0], r_range[1])
            if r_name is not None:
                ax2.set_ylabel(r_name)
        if file_name is not None:
            plt.savefig(file_name, dpi=dpi, bbox_inches='tight')
        plt.close()

    def setImg(self, x_names=range(10), l_range=[0, 10], r_range=None, x_name=None, l_name=None, r_name='', title=None,
               l_compare=[], r_compare=[], dpi=100, font=None, markerSize=7):
        file_name = 'default.png'
        self.draw(x_names, l_range, r_range, x_name, l_name, r_name, title, l_compare, r_compare, file_name, dpi, font,
                  markerSize)
        with open(file_name, 'rb') as imgFile:
            data = imgFile.read()
            self.body_context += '''<img src="%s"></img>''' % (
                'data:default/png;base64,' + str(b64encode(data), 'utf-8'))

    def combine(self):
        self.total_msg = '''<!DOCTYPE html>
            <html>
            <head>
            <title>%s</title>
            <style type="text/css">
            %s
            </style>
            </head>
            <body>
            %s
            </body>
            </html>
            ''' % (self.default_title, self.default_css, self.body_context)

    def save(self, file_name):
        if self.total_msg is None or (isinstance(self.total_msg, str) and self.total_msg == ''):
            self.combine()
        with open(file_name, 'wb') as f:
            f.write(bytes(self.total_msg, 'utf-8'))


if __name__ == '__main__':
    h = Html()
    h.setTable(['a', 'b', 'c'], [['1', '2', '3']], extra=[(2, 0, [h.color_red, h.font_b])])
    h.setH('hello', 4, h.color_blue)
    h.save('a.html')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/2/22 9:49
# @Author  : chenjw
# @Site    : 
# @File    : mysql.py
# @Software: PyCharm Community Edition
# @Desc    :  pymysql need to rely on PooledDB

import pymysql
from DBUtils.PooledDB import PooledDB
import datetime


class Mysql:
    def __init__(self, host, port, user, pwd, db, poolSize=8):
        self.host = host
        self.port = port
        self.user = user
        self.pwd = pwd
        self.db = db
        self.poolSize = poolSize
        self.pool = None

    def connect(self):
        self.pool = PooledDB(pymysql, host=self.host, user=self.user, passwd=self.pwd, db=self.db,
                             port=self.port, charset="utf8", maxconnections=self.poolSize, maxcached=self.poolSize,
                             mincached=self.poolSize)

    def getPool(self):
        if self.pool is None:
            self.connect()
        return self.pool

    def close(self):
        if self.pool is not None:
            try:
                self.pool.close()
            except:
                pass

    def insert(self, sqls):
        '''
        :param sqls: list
        :return:
        '''
        try:
            conn = self.getPool().connection()
            cur = conn.cursor()
            for sql in sqls:
                cur.execute(sql)
            conn.commit()
            return None
        except Exception as e:
            raise Exception(e)
        finally:
            try:
                cur.close()
            except:
                pass
            try:
                conn.close()
            except:
                pass

    def joinValue(self, fields, sep=','):
        ret = ''
        for i in range(len(fields)):
            if fields[i] is None:
                ret += 'NULL'
            elif isinstance(fields[i], str):
                ret += ''' '%s' ''' % fields[i].replace("'", "\\'")
            elif isinstance(fields[i], datetime.datetime):
                ret += ''' '%s' ''' % fields[i]
            else:
                ret += '%s' % fields[i]

            if i != len(fields) - 1:
                ret += sep
        return ret

    def join(self, fields, sep=','):
        '''
        :param fields: ary of field
        :param sep: default ,
        :return: str like field1,field2
        '''
        return sep.join(fields)

    def excute(self, sql, ret=True, commit=False):
        '''
        :param sql: Str
        :param ret: ret or not
        :param commit: call commit or not
        :return:
        '''
        try:
            conn = self.getPool().connection()
            cur = conn.cursor()
            cur.execute(sql)
            if commit:
                conn.commit()
            r = None
            if ret:
                r = cur.fetchall()
            return r
        except Exception as e:
            raise Exception(e)
        finally:
            try:
                cur.close()
            except:
                pass
            try:
                conn.close()
            except:
                pass

    def showCreateTable(self, table):
        '''
        :param table: table name
        :return: str of create table
        '''
        sql = 'show create table %s' % table
        create = ''
        for cell in self.ary2D(self.excute(sql))[0][1:]:
            create += cell + '\n'
        return create

    def showTables(self):
        '''

        :return: ary 1D
        '''
        sql = 'show tables'
        return self.ary1D(self.excute(sql))

    def showField(self, table):
        sql = 'desc %s' % table
        return self.ary1D(self.excute(sql))

    def fieldIdx(self, table):
        sql = 'desc %s' % table
        rsp = self.excute(sql)
        if rsp is None:
            return {}
        else:
            idx = {}
            for single in rsp:
                if len(single) >= 2 and str(single[0]) != '' and str(single[1]) != '':
                    idx[str(single[0])] = str(single[1])
            return {}

    def dropTable(self, table):
        sql = 'drop table %s' % table
        self.excute(sql, False)

    def join(self, fields, sep=','):
        '''
        :param fields: ary of field
        :param sep: default ,
        :return: str like field1,field2
        '''
        ret = sep.join(fields)
        return ret

    def count(self, table):
        sql = 'select count(*) from %s' % table
        return self.ary1D(self.excute(sql))[0]

    def ary1D(self, rsp):
        ret = []
        if rsp is None:
            pass
        else:
            for sin_tuple in rsp:
                if len(sin_tuple) > 0:
                    ret.append(sin_tuple[0])
        return ret

    def ary2D(self, rsp):
        ret = []
        if rsp is None:
            pass
        else:
            for sin_tuple in rsp:
                ary_tmp = []
                for sin in sin_tuple:
                    ary_tmp.append(sin)
                ret.append(ary_tmp)
        return ret


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/5/10 9:35
# @Author  : chenjw
# @Site    : 
# @File    : __init__.py.py
# @Software: PyCharm Community Edition
# @Desc    :  do what

if __name__ == '__main__':
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/5/10 9:35
# @Author  : chenjw
# @Site    : 
# @File    : tapd_param.py
# @Software: PyCharm Community Edition
# @Desc    :  do what


from ccommon import jsonParse
import logging
import datetime

class TapdParam:
    def __init__(self):
        self.notEmpty = 'notEmpty'
        self.lt = 'lt'
        self.lte = 'lte'
        self.gt = 'gt'
        self.gte = 'gte'
        self.Int = 'int'
        self.Str = 'str'
        self.data = {}

    def isParamValid(self, v, eager_type):
        '''
        :param v: any type
        :param eager_type: int,str,dict so os on
        :return:
        '''
        if type(v) == eager_type:
            return True
        else:
            return False

    def setValue(self, v, eager_type, key, isNeed=False, rangeOfValue=None):
        '''
        :param v: any type,value of dict
        :param eager_type: int,str,dict so os on
        :param key: key of dict
        :param data: dict to set key and value
        :return:
        '''
        # 校验参数类型
        if self.isParamValid(v, eager_type):
            # 当没有取值限制的时候直接赋值
            if rangeOfValue is None:
                self.data[key] = v
            elif self.isParamValid(rangeOfValue, dict):
                _json = jsonParse.JsonParse(rangeOfValue)
                # 当 期望类型为 str 时候
                if eager_type == str:
                    if _json.ExistP('%s/%s' % (self.Str, self.notEmpty)) and v == '':
                        logging.warning('call [setValue] key(%s)  v(%s) is notEmpty' % (key, v))
                    else:
                        self.data[key] = v
                # 当 期望类型为 int 时候
                if eager_type == int:
                    int_json = _json.Json(self.Int)
                    normal = True
                    if normal and int_json.Exist(self.lt) and v >= int_json.Int(self.lt):
                        normal = False
                        logging.warning(
                            'call [setValue] eager key(%s) value(%s) to lt(%s)' % (key, v, int_json.Int(self.lt)))
                    if normal and int_json.Exist(self.lte) and v > int_json.Int(self.lte):
                        normal = False
                        logging.warning(
                            'call [setValue] eager key(%s) value(%s) to lte(%s)' % (key, v, int_json.Int(self.lte)))
                    if normal and int_json.Exist(self.gt) and v <= int_json.Int(self.gt):
                        normal = False
                        logging.warning(
                            'call [setValue] eager key(%s) value(%s) to gt(%s)' % (key, v, int_json.Int(self.gt)))
                    if normal and int_json.Exist(self.gte) and v < int_json.Int(self.gte):
                        normal = False
                        logging.warning(
                            'call [setValue] eager key(%s) value(%s) to gte(%s)' % (key, v, int_json.Int(self.gte)))
                    if normal:
                        self.data[key] = v
                if eager_type == datetime.datetime or eager_type == datetime.date:
                    self.data[key] = v

            else:
                logging.warning('call [setValue] rangeOfValue should be dict but not (%s)' % type(rangeOfValue))

        # 当参数为空值时候，且是必传参数时候抛出异常
        elif v is None and isNeed is True:
            logging.warning('call [setValue] key(%s) is Needed but None' % key)
        else:
            logging.debug('call [setValue] key(%s) value(%s) type(%s) will be filter for the eager type is (%s)' % (
                key, v, type(v), eager_type))

if __name__ == '__main__':
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/5/10 9:36
# @Author  : chenjw
# @Site    : 
# @File    : tapd_bug.py
# @Software: PyCharm Community Edition
# @Desc    :  do what


from urllib.parse import urlencode
from ccommon.httpSimple import HttpSimple
from tapdApi import tapd, tapd_param


class TapdBugs(tapd.Tapd):
    def __init__(self, usr, pwd):
        tapd.Tapd.__init__(self, usr, pwd)

    def createBug(self, workspace_id, title, reporter, description=None, de=None, te=None, current_owner=None,
                  status=None, resolution=None, priority=None, severity=None, module=None, version_report=None,
                  version_test=None, test_type=None, originphase=None, source=None, sourcephase=None, cc=None):
        '''
        :param workspace_id:    N   int     所属 TAPD 项目 ID
        :param title:           N   str     标题
        :param description:     O   str     详细描述
        :param reporter:        N   str     提交人
        :param de:              O   str     开发人员
        :param te:              O   str     测试人员
        :param current_owner:   O   str     当前处理人
        :param status:          O   str     状态
        :param resolution:      O   str     解决方法
        :param priority:        O   str     优先级
        :param severity:        O   str     严重程度
        :param module:          O   str     模块
        :param version_report:  O   str     发现版本
        :param version_test:    O   str     验证版本
        :param test_type:       O   str     测试类型
        :param originphase:     O   str     发现阶段
        :param source:          O   str     缺陷根源
        :param sourcephase:     O   str     引入阶段
        :param cc:              O   str     抄送人
        :return:
        '''
        url = 'https://api.tapd.cn/bugs?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(title, str, 'title', True, {param.Str: {param.notEmpty: {}}})
        param.setValue(description, str, 'description')
        param.setValue(reporter, str, 'reporter', True, {param.Str: {param.notEmpty: {}}})
        param.setValue(de, str, 'de')
        param.setValue(te, str, 'te')
        param.setValue(current_owner, str, 'current_owner')
        param.setValue(status, str, 'status')
        param.setValue(resolution, str, 'resolution')
        param.setValue(priority, str, 'priority')
        param.setValue(severity, str, 'severity')
        param.setValue(module, str, 'module')
        param.setValue(version_report, str, 'version_report')
        param.setValue(version_test, str, 'version_test')
        param.setValue(test_type, str, 'test_type')
        param.setValue(originphase, str, 'originphase')
        param.setValue(source, str, 'source')
        param.setValue(sourcephase, str, 'sourcephase')
        param.setValue(cc, str, 'cc')
        self.wait()
        _json = HttpSimple(url).addData(param.data).addMethod(HttpSimple.method_post).addAuth(
            self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        return _json.StrP('data/Bug/id')

    def listBug(self, workspace_id, id=None, reporter=None, title=None, de=None, te=None, current_owner=None,
                status=None, resolution=None, priority=None, severity=None, module=None, version_report=None,
                version_test=None, test_type=None, originphase=None, source=None, sourcephase=None, iteration_id=None,
                closed=None, resolved=None, created=None, modified=None, page=1, limit=30, fields=None):
        '''
        :param workspace_id:    N   int     所属 TAPD 项目 ID
        :param id:              O   int     Bug ID id=1,2,3 支持一次性读取最多 50 个 id 的数据
        :param reporter:        O   str     提交人
        :param title:           O   str     标题 支持模糊匹配
        :param de:              O   str     开发人员 支持模糊匹配
        :param te:              O   str     测试人员 支持模糊匹配
        :param current_owner:   O   str     当前处理人 支持模糊匹配
        :param status:          O   str     状态 支持枚举查询
        :param resolution:      O   str     解决方法 支持枚举查询
        :param priority:        O   str     优先级 支持枚举查询
        :param severity:        O   str     严重程度 支持枚举查询
        :param module:          O   str     模块
        :param version_report:  O   str     发现版本 支持枚举查询
        :param version_test:    O   str     验证版本
        :param test_type:       O   str     测试类型
        :param originphase:     O   str     发现阶段
        :param source:          O   str     缺陷根源
        :param sourcephase:     O   str     引入阶段
        :param iteration_id:    O   str     关联的迭代 id
        :param closed:          O   str     缺陷关闭时间 支持时间字段调用
        :param resolved:        O   str     缺陷解决时间 支持时间字段调用
        :param created:         O   str     创建时间 支持时间字段调用
        :param modified:        O   str     最后修改时间 支持时间字段调用
        :param page:            O   int     设置返回数量限制，默认为 30
        :param limit:           O   int     返回当前数量限制下第 N 页的数据，默认为1（第一页）
        :param fields:          O   str     设置获取的字段，多个字段间以','逗号隔开
        :return:
        '''
        url = 'https://api.tapd.cn/bugs?'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(id, int, 'id')
        param.setValue(reporter, str, 'reporter')
        param.setValue(title, str, 'title')
        param.setValue(de, str, 'de')
        param.setValue(te, str, 'te')
        param.setValue(current_owner, str, 'current_owner')
        param.setValue(status, str, 'status')
        param.setValue(resolution, str, 'resolution')
        param.setValue(priority, str, 'priority')
        param.setValue(severity, str, 'severity')
        param.setValue(module, str, 'module')
        param.setValue(version_report, str, 'version_report')
        param.setValue(version_test, str, 'version_test')
        param.setValue(test_type, str, 'test_type')
        param.setValue(originphase, str, 'originphase')
        param.setValue(source, str, 'source')
        param.setValue(sourcephase, str, 'sourcephase')
        param.setValue(iteration_id, str, 'iteration_id')
        param.setValue(closed, str, 'closed')
        param.setValue(resolved, str, 'resolved')
        param.setValue(created, str, 'created')
        param.setValue(modified, str, 'modified')
        param.setValue(limit, int, 'limit', False, {param.Int: {param.gt: 0}})
        param.setValue(page, int, 'page', False, {param.Int: {param.gt: 0, param.lte: 30}})
        param.setValue(fields, str, 'fields')
        self.wait()
        url = '%s%s' % (url, urlencode(param.data))

        _json = HttpSimple(url).addMethod(HttpSimple.method_get).addAuth(self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        if _json.ExistP('data/Bug/id'):
            return _json.JsonP('data/Bug').Dict
        elif _json.ExistP('data/0/Bug/id'):
            return [ain_json.Dict for ain_json in _json.AryJson('data')]
        else:
            return None

    def updateBug(self, workspace_id, id, current_user, title=None, description=None, reporter=None, de=None, te=None,
                  status=None, resolution=None, priority=None, severity=None, module=None, version_report=None,
                  version_test=None, test_type=None, originphase=None, source=None, sourcephase=None):
        '''
        :param workspace_id:    N   int     所属 TAPD 项目 ID
        :param id:Bug ID:       N   int     Bug ID
        :param current_user:    N   str     当前处理人
        :param title:           O   str     标题
        :param description:     O   str     详细描述
        :param reporter:        O   str     提交人
        :param de:              O   str     开发人员
        :param te:              O   str     测试人员
        :param status:          O   str     状态
        :param resolution:      O   str     解决方法
        :param priority:        O   str     优先级
        :param severity:        O   str     严重程度
        :param module:          O   str     模块
        :param version_report:  O   str     发现版本
        :param version_test:    O   str     验证版本
        :param test_type:       O   str     测试类型
        :param originphase:     O   str     发现阶段
        :param source:          O   str     缺陷根源
        :param sourcephase:     O   str     引入阶段
        :return:
        '''
        url = 'https://api.tapd.cn/bugs'
        param = tapd_param.TapdParam()
        param.setValue(workspace_id, int, 'workspace_id', True, {param.Int: {param.gt: 0}})
        param.setValue(id, int, 'id', True, {param.Int: {param.gt: 0}})
        param.setValue(current_user, str, 'current_user', True, {param.Str: {param.notEmpty: {}}})
        param.setValue(title, str, 'title')
        param.setValue(description, str, 'description')
        param.setValue(reporter, str, 'reporter')
        param.setValue(de, str, 'de')
        param.setValue(te, str, 'te')
        param.setValue(status, str, 'status')
        param.setValue(resolution, str, 'resolution')
        param.setValue(priority, str, 'priority')
        param.setValue(severity, str, 'severity')
        param.setValue(module, str, 'module')
        param.setValue(version_report, str, 'version_report')
        param.setValue(version_test, str, 'version_test')
        param.setValue(test_type, str, 'test_type')
        param.setValue(originphase, str, 'originphase')
        param.setValue(source, str, 'source')
        param.setValue(sourcephase, str, 'sourcephase')
        self.wait()
        _json = HttpSimple(url).addData(param.data).addMethod(HttpSimple.method_post).addAuth(
            self.retAuth()).run().retJson()
        self.checkApiStatus(_json)
        return _json.JsonP('ary_data/Bug')


if __name__ == '__main__':
    pass
